//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef FACTOR_COMPASS_3D_H_
#define FACTOR_COMPASS_3D_H_

#include "core/factor/factor_autodiff.h"

namespace wolf
{

WOLF_PTR_TYPEDEFS(FactorCompass3d);

class FactorCompass3d : public FactorAutodiff<FactorCompass3d, 3, 4, 4, 3, 3>
{
    public:
        FactorCompass3d(const FeatureBasePtr&   _feat,
                        const ProcessorBasePtr& _processor_ptr,
                        bool                    _apply_loss_function,
                        FactorStatus            _status = FAC_ACTIVE) :
            FactorAutodiff("FactorCompass3d",
                           TOP_ABS,
                           _feat,
                           nullptr,
                           nullptr,
                           nullptr,
                           nullptr,
                           _processor_ptr,
                           _apply_loss_function,
                           _status,
                           _feat->getFrame()->getO(),
                           _feat->getCapture()->getSensor()->getO(),
                           _feat->getCapture()->getSensor()->getIntrinsic(),
                           _feat->getCapture()->getSensor()->getStateBlock('F'))
        {
        }

        ~FactorCompass3d() override { /* nothing */ }

        template<typename T>
        bool operator () (const T* const _frm_q,
                          const T* const _sen_q,
                          const T* const _sen_bias,
                          const T* const _field,
                          T* _residual) const
        {
            using namespace Eigen;

            Map<const Quaternion<T>> frm_q(_frm_q);
            Map<const Quaternion<T>> sen_q(_sen_q);
            Map<const Matrix<T,3,1>> sen_bias(_sen_bias);
            Map<const Matrix<T,3,1>> field(_field);
            Map<Matrix<T,3,1>> res(_residual);

            Matrix<T,3,1> exp = sen_q.conjugate() * frm_q.conjugate() * field + sen_bias;

            res  = getMeasurementSquareRootInformationUpper() * (getMeasurement() - exp);

            return true;
        }
};

} /* namespace wolf */

#endif /* FACTOR_COMPASS_3D_H_ */
