//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------

#ifndef FACTOR_FIX_BIAS_H_
#define FACTOR_FIX_BIAS_H_

//Wolf includes
#include "imu/capture/capture_imu.h"
#include "imu/feature/feature_imu.h"
#include "core/factor/factor_autodiff.h"
#include "core/frame/frame_base.h"
#include "core/math/rotations.h"

//#include "ceres/jet.h"

namespace wolf {
    
WOLF_PTR_TYPEDEFS(FactorFixBias);

//class
class FactorFixBias: public FactorAutodiff<FactorFixBias,6,3,3>
{
    public:
        FactorFixBias(FeatureBasePtr _ftr_ptr, bool _apply_loss_function, FactorStatus _status = FAC_ACTIVE) :
                FactorAutodiff<FactorFixBias, 6, 3, 3>("FactorFixBias",
                                                       TOP_ABS,
                                                       _ftr_ptr,
                                                       nullptr, nullptr, nullptr, nullptr,
                                                       nullptr,
                                                       _apply_loss_function,
                                                       _status,
                                                       std::static_pointer_cast<CaptureImu>(_ftr_ptr->getCapture())->getAccBias(),
                                                       std::static_pointer_cast<CaptureImu>(_ftr_ptr->getCapture())->getGyroBias())
        {
            // std::cout << "created FactorFixBias " << std::endl;
        }

        virtual ~FactorFixBias() = default;

        template<typename T>
        bool operator ()(const T* const _ab, const T* const _wb, T* _residuals) const;

};

template<typename T>
inline bool FactorFixBias::operator ()(const T* const _ab, const T* const _wb, T* _residuals) const
{
    // measurement
    Eigen::Matrix<T,6,1> meas =  getMeasurement().cast<T>();
    Eigen::Matrix<T,3,1> ab(_ab);
    Eigen::Matrix<T,3,1> wb(_wb);

    // error
    Eigen::Matrix<T,6,1> er;
    er.head(3) = meas.head(3) - ab;
    er.tail(3) = meas.tail(3) - wb;

    // residual
    Eigen::Map<Eigen::Matrix<T,6,1>> res(_residuals);
    res = getMeasurementSquareRootInformationUpper().cast<T>() * er;

    ////////////////////////////////////////////////////////
    // print Jacobian. Uncomment this as you wish (remember to uncomment #include "ceres/jet.h" above):
//    using ceres::Jet;
//    Eigen::MatrixXd J(6,6);
//    J.row(0) = ((Jet<double, 3>)(er(0))).v;
//    J.row(1) = ((Jet<double, 3>)(er(1))).v;
//    J.row(2) = ((Jet<double, 3>)(er(2))).v;
//    J.row(3) = ((Jet<double, 3>)(er(3))).v;
//    J.row(4) = ((Jet<double, 3>)(er(4))).v;
//    J.row(5) = ((Jet<double, 3>)(er(5))).v;

//    J.row(0) = ((Jet<double, 3>)(res(0))).v;
//    J.row(1) = ((Jet<double, 3>)(res(1))).v;
//    J.row(2) = ((Jet<double, 3>)(res(2))).v;
//    J.row(3) = ((Jet<double, 3>)(res(3))).v;
//    J.row(4) = ((Jet<double, 3>)(res(4))).v;
//    J.row(5) = ((Jet<double, 3>)(res(5))).v;

//    if (sizeof(er(0)) != sizeof(double))
//    {
//        std::cout << "FactorFixBias::Jacobian(c" << id() << ") = \n " << J << std::endl;
//        std::cout << "FactorFixBias::Weighted Jacobian(c" << id() << ") = \n " << J << std::endl;
//        std::cout << "Sqrt Info(c" << id() << ") = \n " << getMeasurementSquareRootInformationUpper() << std::endl;
//    }
    ////////////////////////////////////////////////////////

    return true;
}

} // namespace wolf

#endif
