//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef FEATURE_IMU2D_H_
#define FEATURE_IMU2D_H_

//Wolf includes
#include "imu/capture/capture_imu.h"
#include "core/feature/feature_base.h"
#include "core/common/wolf.h"

//std includes

namespace wolf {

WOLF_PTR_TYPEDEFS(FeatureImu2d);

class FeatureImu2d : public FeatureBase
{
    public:

        /** \brief Constructor from and measures
         *
         * \param _measurement the measurement
         * \param _meas_covariance the noise of the measurement
         * \param _dD_db_jacobians Jacobians of preintegrated delta wrt Imu2d biases
         * \param acc_bias accelerometer bias of origin frame
         * \param gyro_bias gyroscope bias of origin frame
         * \param _cap_imu_ptr pointer to parent CaptureMotion
         */
        FeatureImu2d(const Eigen::VectorXd& _delta_preintegrated,
                   const Eigen::MatrixXd& _delta_preintegrated_covariance,
                   const Eigen::Vector3d& _bias,
                   const Eigen::Matrix<double,5,3>& _dD_db_jacobians,
                   CaptureMotionPtr _cap_imu_ptr = nullptr);

        /** \brief Constructor from capture pointer
         *
         * \param _cap_imu_ptr pointer to parent CaptureMotion
         */
        FeatureImu2d(CaptureMotionPtr _cap_imu_ptr);

        ~FeatureImu2d() override;

        const Eigen::Vector3d&              getBiasPreint()  const;
        const Eigen::Matrix<double, 5, 3>&  getJacobianBias()   const;

    private:

        // Used biases
        Eigen::Vector3d bias_preint_;       

        Eigen::Matrix<double, 5, 3> jacobian_bias_;

    public:
      EIGEN_MAKE_ALIGNED_OPERATOR_NEW;
};

inline const Eigen::Vector3d& FeatureImu2d::getBiasPreint() const
{
    return bias_preint_;
}

inline const Eigen::Matrix<double, 5, 3>& FeatureImu2d::getJacobianBias() const
{
    return jacobian_bias_;
}

} // namespace wolf

#endif
