//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef INCLUDE_IMU_PROCESSOR_PROCESSORCOMPASS_H_
#define INCLUDE_IMU_PROCESSOR_PROCESSORCOMPASS_H_

#include "core/processor/processor_base.h"

namespace wolf {

WOLF_STRUCT_PTR_TYPEDEFS(ParamsProcessorCompass);

struct ParamsProcessorCompass : public ParamsProcessorBase
{
    ParamsProcessorCompass() = default;
    ParamsProcessorCompass(std::string _unique_name, const ParamsServer& _server):
        ParamsProcessorBase(_unique_name, _server)
    {
        //
    }
    std::string print() const override
    {
        return "\n" + ParamsProcessorBase::print();
    }
};

WOLF_PTR_TYPEDEFS(ProcessorCompass);

//class
class ProcessorCompass : public ProcessorBase
{
    protected:
        ParamsProcessorCompassPtr params_compass_;

    public:
        ProcessorCompass(ParamsProcessorCompassPtr);
        ~ProcessorCompass() override;
        void configure(SensorBasePtr) override { };

        WOLF_PROCESSOR_CREATE(ProcessorCompass, ParamsProcessorCompass);

    protected:
        void processCapture(CaptureBasePtr) override;
        void processKeyFrame(FrameBasePtr _frm) override;
        void processMatch(FrameBasePtr, CaptureBasePtr);

        bool triggerInCapture(CaptureBasePtr _cap) const override { return true;};
        bool triggerInKeyFrame(FrameBasePtr _frm) const override { return true;};

        bool storeKeyFrame(FrameBasePtr _frm) override { return false;};
        bool storeCapture(CaptureBasePtr _cap) override { return false;};

        bool voteForKeyFrame() const override { return false;};
};

} /* namespace wolf */

#endif /* INCLUDE_IMU_PROCESSOR_PROCESSORCOMPASS_H_ */
