//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef PROCESSOR_IMU_H
#define PROCESSOR_IMU_H

// Wolf
#include "imu/capture/capture_imu.h"
#include "imu/feature/feature_imu.h"
#include "core/processor/processor_motion.h"

namespace wolf {
WOLF_STRUCT_PTR_TYPEDEFS(ParamsProcessorImu);

struct ParamsProcessorImu : public ParamsProcessorMotion
{
    ParamsProcessorImu() = default;
    ParamsProcessorImu(std::string _unique_name, const ParamsServer& _server):
        ParamsProcessorMotion(_unique_name, _server)
    {
        //
    }
    std::string print() const override
    {
        return ParamsProcessorMotion::print();
    }
};

WOLF_PTR_TYPEDEFS(ProcessorImu);

//class
class ProcessorImu : public ProcessorMotion{
    public:
        ProcessorImu(ParamsProcessorImuPtr _params_motion_Imu);
        ~ProcessorImu() override;
        void configure(SensorBasePtr _sensor) override { };

        WOLF_PROCESSOR_CREATE(ProcessorImu, ParamsProcessorImu);
        void preProcess() override;

    protected:
        void computeCurrentDelta(const Eigen::VectorXd& _data,
                                         const Eigen::MatrixXd& _data_cov,
                                         const Eigen::VectorXd& _calib,
                                         const double _dt,
                                         Eigen::VectorXd& _delta,
                                         Eigen::MatrixXd& _delta_cov,
                                         Eigen::MatrixXd& _jacobian_calib) const override;
        void deltaPlusDelta(const Eigen::VectorXd& _delta_preint,
                                    const Eigen::VectorXd& _delta,
                                    const double _dt,
                                    Eigen::VectorXd& _delta_preint_plus_delta) const override;
        void deltaPlusDelta(const Eigen::VectorXd& _delta_preint,
                                    const Eigen::VectorXd& _delta,
                                    const double _dt,
                                    Eigen::VectorXd& _delta_preint_plus_delta,
                                    Eigen::MatrixXd& _jacobian_delta_preint,
                                    Eigen::MatrixXd& _jacobian_delta) const override;
        void statePlusDelta(const VectorComposite& _x,
                                    const Eigen::VectorXd& _delta,
                                    const double _Dt,
                                    VectorComposite& _x_plus_delta) const override;
        Eigen::VectorXd deltaZero() const override;
        Eigen::VectorXd correctDelta(const Eigen::VectorXd& delta_preint,
                                             const Eigen::VectorXd& delta_step) const override;
        VectorXd getCalibration (const CaptureBasePtr _capture = nullptr) const override;
        void setCalibration(const CaptureBasePtr _capture, const VectorXd& _calibration) override;
        bool voteForKeyFrame() const override;
        CaptureMotionPtr emplaceCapture(const FrameBasePtr& _frame_own,
                                                const SensorBasePtr& _sensor,
                                                const TimeStamp& _ts,
                                                const VectorXd& _data,
                                                const MatrixXd& _data_cov,
                                                const VectorXd& _calib,
                                                const VectorXd& _calib_preint,
                                                const CaptureBasePtr& _capture_origin) override;
        FeatureBasePtr emplaceFeature(CaptureMotionPtr _capture_motion) override;
        FactorBasePtr emplaceFactor(FeatureBasePtr _feature_motion,
                                            CaptureBasePtr _capture_origin) override;

    protected:
        ParamsProcessorImuPtr params_motion_Imu_;
};

}

/////////////////////////////////////////////////////////
// IMPLEMENTATION. Put your implementation includes here
/////////////////////////////////////////////////////////

namespace wolf{

inline Eigen::VectorXd ProcessorImu::deltaZero() const
{
    return (Eigen::VectorXd(10) << 0,0,0,  0,0,0,1,  0,0,0 ).finished(); // p, q, v
}

} // namespace wolf

#endif // PROCESSOR_Imu_H
