//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef SENSOR_SENSOR_COMPASS_H_
#define SENSOR_SENSOR_COMPASS_H_

//wolf includes
#include "core/sensor/sensor_base.h"
#include "core/utils/params_server.h"
#include <iostream>

namespace wolf {

WOLF_STRUCT_PTR_TYPEDEFS(ParamsSensorCompass);

struct ParamsSensorCompass : public ParamsSensorBase
{
    double stdev_noise;

    std::string     field_prior_mode;
    Eigen::Vector3d field_prior_state;
    Eigen::Vector3d field_prior_sigma;

    std::string     bias_prior_mode;
    Eigen::Vector3d bias_prior_state;
    Eigen::Vector3d bias_prior_sigma;

    ParamsSensorCompass() = default;
    ParamsSensorCompass(std::string _unique_name, const wolf::ParamsServer& _server)
      : ParamsSensorBase(_unique_name, _server)
    {
        stdev_noise         = _server.getParam<double>          (prefix + _unique_name + "/stdev_noise");

        field_prior_mode    = _server.getParam<std::string>     (prefix + _unique_name + "/field_prior_mode");
        field_prior_state   = _server.getParam<Eigen::Vector3d> (prefix + _unique_name + "/field_prior_state");
        if (field_prior_mode == "factor")
            field_prior_sigma = _server.getParam<Eigen::Vector3d> (prefix + _unique_name + "/field_prior_sigma");

        bias_prior_mode    = _server.getParam<std::string>     (prefix + _unique_name + "/bias_prior_mode");
        bias_prior_state   = _server.getParam<Eigen::Vector3d> (prefix + _unique_name + "/bias_prior_state");
        if (bias_prior_mode == "factor")
            bias_prior_sigma = _server.getParam<Eigen::Vector3d> (prefix + _unique_name + "/bias_prior_sigma");
    }
    ~ParamsSensorCompass() override = default;
    std::string print() const override
    {
        return ParamsSensorBase::print()  + "\n"
        + "stdev_noise: "       + std::to_string(stdev_noise)       + "\n"
        + "field_prior_mode: "  + field_prior_mode                  + "\n"
        + "field_prior_state: print not implemented. \n"
        + "field_prior_sigma: print not implemented. \n"
        + "bias_prior_mode: "   + bias_prior_mode                   + "\n"
        + "bias_prior_state:  print not implemented. \n"
        + "bias_prior_sigma:  print not implemented. \n";
    }
};

WOLF_PTR_TYPEDEFS(SensorCompass);

class SensorCompass : public SensorBase
{
    public:
        SensorCompass(const Eigen::VectorXd& _extrinsics, ParamsSensorCompassPtr _params);
        SensorCompass(const Eigen::VectorXd& _extrinsics, const ParamsSensorCompass& _params);
        WOLF_SENSOR_CREATE(SensorCompass, ParamsSensorCompass, 7);

        ~SensorCompass() override;
        ParamsSensorCompassConstPtr getParams() const;

        Eigen::Matrix3d computeMeasurementCovariance(const Eigen::Vector3d& _field) const;

    protected:
        ParamsSensorCompassPtr params_compass_;
};

inline ParamsSensorCompassConstPtr SensorCompass::getParams() const
{
    return params_compass_;
}

} /* namespace wolf */

#endif /* SENSOR_SENSOR_DIFF_DRIVE_H_ */
