//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef INCLUDE_TREE_MANAGER_SLIDING_WINDOW_STATIC_BIAS_H_
#define INCLUDE_TREE_MANAGER_SLIDING_WINDOW_STATIC_BIAS_H_

#include "core/tree_manager/tree_manager_sliding_window.h"

namespace wolf
{

WOLF_STRUCT_PTR_TYPEDEFS(ParamsTreeManagerSlidingWindowStaticBias)
WOLF_PTR_TYPEDEFS(TreeManagerSlidingWindowStaticBias)

struct ParamsTreeManagerSlidingWindowStaticBias : public ParamsTreeManagerSlidingWindow
{
        ParamsTreeManagerSlidingWindowStaticBias() = default;
        ParamsTreeManagerSlidingWindowStaticBias(std::string _unique_name, const wolf::ParamsServer & _server) :
            ParamsTreeManagerSlidingWindow(_unique_name, _server)
        {
            sensor_name = _server.getParam<std::string>(prefix + "/sensor_name");
            ab_stdev        = _server.getParam<double>(prefix + "/ab_stdev");
            wb_stdev        = _server.getParam<double>(prefix + "/wb_stdev");
        }
        std::string print() const override
        {
            return "\n" + ParamsTreeManagerSlidingWindow::print()       + "\n"
                        + "sensor_name: "              + sensor_name    + "\n";
        }

        std::string sensor_name;
        double ab_stdev, wb_stdev;
};

class TreeManagerSlidingWindowStaticBias : public TreeManagerSlidingWindow
{
    public:
        TreeManagerSlidingWindowStaticBias(ParamsTreeManagerSlidingWindowStaticBiasPtr _params);
        WOLF_TREE_MANAGER_CREATE(TreeManagerSlidingWindowStaticBias, ParamsTreeManagerSlidingWindowStaticBias)

        ~TreeManagerSlidingWindowStaticBias() override{}

        void keyFrameCallback(FrameBasePtr _key_frame) override;

    protected:
        ParamsTreeManagerSlidingWindowStaticBiasPtr params_sw_sb_;
        SensorBasePtr sensor_imu_;
        Eigen::Matrix6d cov_bias_;
};

} /* namespace wolf */

#endif /* INCLUDE_TREE_MANAGER_SLIDING_WINDOW_H_ */
