//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "imu/capture/capture_imu.h"
#include "imu/sensor/sensor_imu.h"
#include "core/state_block/state_quaternion.h"

namespace wolf {

CaptureImu::CaptureImu(const TimeStamp& _init_ts,
                       SensorBasePtr _sensor_ptr,
                       const Eigen::Vector6d& _acc_gyro_data,
                       const Eigen::MatrixXd& _data_cov,
                       CaptureBasePtr _capture_origin_ptr) :
                CaptureMotion("CaptureImu",
                              _init_ts,
                              _sensor_ptr,
                              _acc_gyro_data,
                              _data_cov,
                              _capture_origin_ptr,
                              nullptr,
                              nullptr,
                              _sensor_ptr->isStateBlockDynamic('I') ? std::make_shared<StateBlock>((_sensor_ptr->getProblem()->getDim() == 2 ? 3 : 6), false) : nullptr)
{
    //
}

CaptureImu::CaptureImu(const TimeStamp& _init_ts,
                       SensorBasePtr _sensor_ptr,
                       const Eigen::Vector6d& _acc_gyro_data,
                       const Eigen::MatrixXd& _data_cov,
                       const VectorXd& _bias,
                       CaptureBasePtr _capture_origin_ptr) :
                CaptureMotion("CaptureImu",
                              _init_ts,
                              _sensor_ptr,
                              _acc_gyro_data,
                              _data_cov,
                              _capture_origin_ptr,
                              nullptr,
                              nullptr,
                              _sensor_ptr->isStateBlockDynamic('I') ? std::make_shared<StateBlock>(_bias, false) : nullptr)
{
    assert((_bias.size() == 3) or (_bias.size() == 6));
}

CaptureImu::~CaptureImu()
{
    //
}

} //namespace wolf
