//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "imu/feature/feature_imu.h"

namespace wolf {

FeatureImu::FeatureImu(const Eigen::VectorXd& _delta_preintegrated,
                       const Eigen::MatrixXd& _delta_preintegrated_covariance,
                       const Eigen::Vector6d& _bias,
                       const Eigen::Matrix<double,9,6>& _dD_db_jacobians,
                       CaptureMotionPtr _cap_imu_ptr) :
    FeatureBase("FeatureImu", _delta_preintegrated, _delta_preintegrated_covariance),
    acc_bias_preint_(_bias.head<3>()),
    gyro_bias_preint_(_bias.tail<3>()),
    jacobian_bias_(_dD_db_jacobians)
{
}

FeatureImu::FeatureImu(CaptureMotionPtr _cap_imu_ptr):
        FeatureBase("FeatureImu", _cap_imu_ptr->getDeltaPreint(), _cap_imu_ptr->getDeltaPreintCov()),
        acc_bias_preint_ (_cap_imu_ptr->getCalibrationPreint().head<3>()),
        gyro_bias_preint_(_cap_imu_ptr->getCalibrationPreint().tail<3>()),
        jacobian_bias_(_cap_imu_ptr->getJacobianCalib())
{
}

FeatureImu::~FeatureImu()
{
    //
}

} // namespace wolf
