//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "imu/sensor/sensor_imu.h"
#include "core/state_block/state_block.h"
#include "core/state_block/state_quaternion.h"

namespace wolf {

SensorImu::SensorImu(const Eigen::VectorXd& _extrinsics, ParamsSensorImuPtr _params) :
        SensorImu(_extrinsics, *_params)
{
    //
}

SensorImu::SensorImu(const Eigen::VectorXd& _extrinsics, const ParamsSensorImu& _params) :
        SensorBase("SensorImu",
                   std::make_shared<StateBlock>(_extrinsics.head(3), true),
                   std::make_shared<StateQuaternion>(_extrinsics.tail(4), true),
                   std::make_shared<StateBlock>(6, false, nullptr),
                   (Eigen::Vector6d()<<_params.a_noise,_params.a_noise,_params.a_noise,_params.w_noise,_params.w_noise,_params.w_noise).finished(),
                   false,
                   false,
                   _params.bias_dynamic),
        a_noise(_params.a_noise),
        w_noise(_params.w_noise),
        prior_bias(_params.prior_bias),
        ab_initial_stdev(_params.ab_initial_stdev),
        wb_initial_stdev(_params.wb_initial_stdev),
        bias_dynamic(_params.bias_dynamic),
        ab_rate_stdev(_params.ab_rate_stdev),
        wb_rate_stdev(_params.wb_rate_stdev)
{
    assert(_extrinsics.size() == 7 && "Wrong extrinsics vector size! Should be 7 for 3d.");

    if (prior_bias)
    {
        addPriorIntrinsics(Eigen::Vector6d::Zero(),
                           (Eigen::Vector6d() << ab_initial_stdev*ab_initial_stdev,
                                                 ab_initial_stdev*ab_initial_stdev,
                                                 ab_initial_stdev*ab_initial_stdev,
                                                 wb_initial_stdev*wb_initial_stdev,
                                                 wb_initial_stdev*wb_initial_stdev,
                                                 wb_initial_stdev*wb_initial_stdev).finished().asDiagonal());
        //getIntrinsic()->fix();
    }
}

SensorImu::~SensorImu()
{
    //
}

} // namespace wolf

// Register in the FactorySensor
#include "core/sensor/factory_sensor.h"
namespace wolf {
WOLF_REGISTER_SENSOR(SensorImu)
WOLF_REGISTER_SENSOR_AUTO(SensorImu);
} // namespace wolf
