//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include <imu/sensor/sensor_imu2d.h>
#include <core/state_block/state_block.h>
#include <core/state_block/state_angle.h>

namespace wolf {

SensorImu2d::SensorImu2d(const Eigen::VectorXd& _extrinsics, ParamsSensorImu2dPtr _params) :
        SensorImu2d(_extrinsics, *_params)
{
    //
}

SensorImu2d::SensorImu2d(const Eigen::VectorXd& _extrinsics, const ParamsSensorImu2d& _params) :
        SensorBase("SensorImu2d", std::make_shared<StateBlock>(_extrinsics.head(2), true), std::make_shared<StateAngle>(_extrinsics(2), true), std::make_shared<StateBlock>(3, false, nullptr), (Eigen::Vector3d()<<_params.a_noise,_params.a_noise,_params.w_noise).finished(), false, false, true),
        a_noise(_params.a_noise),
        w_noise(_params.w_noise),
        ab_initial_stdev(_params.ab_initial_stdev),
        wb_initial_stdev(_params.wb_initial_stdev),
        ab_rate_stdev(_params.ab_rate_stdev),
        wb_rate_stdev(_params.wb_rate_stdev)
{
    assert(_extrinsics.size() == 3 && "Wrong extrinsics vector size! Should be 3 for 2d.");
}

SensorImu2d::~SensorImu2d()
{
    //
}

} // namespace wolf

// Register in the FactorySensor
#include "core/sensor/factory_sensor.h"
namespace wolf {
WOLF_REGISTER_SENSOR(SensorImu2d)
WOLF_REGISTER_SENSOR_AUTO(SensorImu2d);
} // namespace wolf
