//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "imu/tree_manager/tree_manager_sliding_window_static_bias.h"
#include "imu/sensor/sensor_imu.h"

namespace wolf
{

TreeManagerSlidingWindowStaticBias::TreeManagerSlidingWindowStaticBias(ParamsTreeManagerSlidingWindowStaticBiasPtr _params) :
            TreeManagerSlidingWindow(_params),
            params_sw_sb_(_params)
{
    NodeBase::setType("TreeManagerSlidingWindowStaticBias");

    cov_bias_ = (Eigen::Vector6d() << params_sw_sb_->ab_stdev*params_sw_sb_->ab_stdev,
                                      params_sw_sb_->ab_stdev*params_sw_sb_->ab_stdev,
                                      params_sw_sb_->ab_stdev*params_sw_sb_->ab_stdev,
                                      params_sw_sb_->wb_stdev*params_sw_sb_->wb_stdev,
                                      params_sw_sb_->wb_stdev*params_sw_sb_->wb_stdev,
                                      params_sw_sb_->wb_stdev*params_sw_sb_->wb_stdev).finished().asDiagonal();
};

void TreeManagerSlidingWindowStaticBias::keyFrameCallback(FrameBasePtr _key_frame)
{
    assert(getProblem() && "TreeManagerSlidingWindowStaticBias::keyFrameCallback: problem not set.");

    // init sensor_imu_
    if (sensor_imu_ == nullptr)
    {
        sensor_imu_ = getProblem()->getSensor(params_sw_sb_->sensor_name);
        assert(sensor_imu_ && "TreeManagerSlidingWindowStaticBias::keyFrameCallback: sensor name didn't match with any sensor.");
        assert(std::dynamic_pointer_cast<SensorImu>(sensor_imu_) && "TreeManagerSlidingWindowStaticBias::keyFrameCallback: sensor name does not correspond to a SensorImu.");
    }

    // copy current value of bias
    auto state_bias = sensor_imu_->getIntrinsic()->getState();

    // call base sliding window tree manager
    TreeManagerSlidingWindow::keyFrameCallback(_key_frame);

    // add new bias prior with stored value (it removes other prior factors)
    sensor_imu_->addPriorIntrinsics(state_bias, cov_bias_);
}

} /* namespace wolf */

// Register in the FactoryTreeManager
#include "core/tree_manager/factory_tree_manager.h"
namespace wolf {
WOLF_REGISTER_TREE_MANAGER(TreeManagerSlidingWindowStaticBias);
WOLF_REGISTER_TREE_MANAGER_AUTO(TreeManagerSlidingWindowStaticBias);
} // namespace wolf

