//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
/**
 * \file processor_imu2d_yaml.cpp
 *
 *  Created on: Nov 24, 2020
 *      \author: igeer
 */

// wolf yaml
#include "imu/processor/processor_imu2d.h"
#include "core/yaml/yaml_conversion.h"

// wolf
#include "core/common/factory.h"

// yaml-cpp library
#include <yaml-cpp/yaml.h>

namespace wolf
{

namespace
{
static ParamsProcessorBasePtr createProcessorImu2dParams(const std::string & _filename_dot_yaml)
{
    YAML::Node config = YAML::LoadFile(_filename_dot_yaml);
    std::cout << _filename_dot_yaml << '\n';

    if (config["type"].as<std::string>() == "ProcessorImu2d")
    {
        YAML::Node kf_vote = config["keyframe_vote"];

        ParamsProcessorImu2dPtr params = std::make_shared<ParamsProcessorImu2d>();
        params->time_tolerance = config["time_tolerance"]           .as<double>();
        params->unmeasured_perturbation_std = config["unmeasured_perturbation_std"].as<double>();

        params->max_time_span       = kf_vote["max_time_span"]      .as<double>();
        params->max_buff_length     = kf_vote["max_buff_length"]    .as<SizeEigen>();
        params->dist_traveled       = kf_vote["dist_traveled"]      .as<double>();
        params->angle_turned        = kf_vote["angle_turned"]       .as<double>();
        params->voting_active       = kf_vote["voting_active"]      .as<bool>();
        return params;
    }

    std::cout << "Bad configuration file. No processor type found." << std::endl;
    return nullptr;
}

// Register in the FactorySensor
const bool WOLF_UNUSED registered_prc_imu2d = FactoryParamsProcessor::registerCreator("ProcessorImu2d", createProcessorImu2dParams);

} // namespace [unnamed]

} // namespace wolf
