//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
/**
 * \file sensor_imu_yaml.cpp
 *
 *  Created on: Jan 18, 2017
 *      \author: Dinesh Atchuthan
 */

// wolf yaml
#include "imu/sensor/sensor_imu.h"
#include "core/yaml/yaml_conversion.h"

// wolf
#include "core/common/factory.h"

// yaml-cpp library
#include <yaml-cpp/yaml.h>

namespace wolf
{

namespace
{

static ParamsSensorBasePtr createParamsSensorImu(const std::string & _filename_dot_yaml)
{
    YAML::Node config = YAML::LoadFile(_filename_dot_yaml);

    if (config["type"].as<std::string>() == "SensorImu")
    {
        YAML::Node variances        = config["motion_variances"];

        ParamsSensorImuPtr params = std::make_shared<ParamsSensorImu>();

        params->a_noise             = variances["a_noise"]          .as<double>();
        params->w_noise             = variances["w_noise"]          .as<double>();
        params->ab_initial_stdev    = variances["ab_initial_stdev"] .as<double>();
        params->wb_initial_stdev    = variances["wb_initial_stdev"] .as<double>();
        params->ab_rate_stdev       = variances["ab_rate_stdev"]    .as<double>();
        params->wb_rate_stdev       = variances["wb_rate_stdev"]    .as<double>();

        return params;
    }

    std::cout << "Bad configuration file. No sensor type found." << std::endl;
    return nullptr;
}

// Register in the FactorySensor
const bool WOLF_UNUSED registered_params_imu = FactoryParamsSensor::registerCreator("SensorImu", createParamsSensorImu);

} // namespace [unnamed]

} // namespace wolf
